#ifndef __KSEQAPI_H
#define __KSEQAPI_H

#ifdef KSEQ_EXPORTS
#define KSEQ_API __declspec(dllexport)
#else
#define KSEQ_API __declspec(dllimport)
#endif

#ifdef __cplusplus
extern "C" {
#endif

typedef void* HKSEQ;

/*
** CreateKSeqHandler - Creates an uninitialized KSequance handler.
**
** ExposeIO: TRUE to enable exposure of IO (X, Y, WX, WY, DWX, DWY).
**
** Note: Handler must be initialized with InitAsXXXXHandler before use.
*/
KSEQ_API HKSEQ CreateKSeqHandler(BOOL ExposeIO);

/*
** DestroyKSeqHandler - Destroys a previously created KSeq handler.
**
** hKSeq: Handle to KSeq handler to destroy
*/
KSEQ_API void DestroyKSeqHandler(HKSEQ hKSeq);

/*
** InitAsSerialHandler - Initializes a KSeq handler for use with serial port.
**
** hKSeq: Handle of KSeq to initialize.
** pPort: Pointer to wide string containing port to open. Example L"COM1:". Use L"COM0:" for COM10.
** BaudRate: Baud rate of port. Uses standard Win32 defines. Example CBR_9600. See CBR_XXX defines in Win32 docs.
** ByteSize: Size of data byte. Normally 8.
** Parity: Parity of port. Uses standard Win32 defines. Example NOPARITY.
** StopBits: Number of stop bits for port. Uses standard Win32 defines. Example ONESTOPBIT.
**
** To emulate a DL250 programming port: InitAsSerialHandler(hKSeq, L"COM1:", CBR_9600, 8, ODDPARITY, ONESTOPBIT);
**
*/
KSEQ_API BOOL InitAsSerialHandler(HKSEQ hKSeq, WCHAR *pPort, DWORD BaudRate, BYTE ByteSize, BYTE Parity, BYTE StopBits);

/*
** InitAsEthernetHandler - Initializes a KSeq handler for use with ethernet port.
**
** hKSeq: Handle of KSeq to initialize.
** Port: IP port value. Do not use Host default value of 0x7070. Anything above decimal 5000 is fine. 
*/
KSEQ_API BOOL InitAsEthernetHandler(HKSEQ hKSeq, WORD Port);

/*
** ExitHandler - Terminates KSeq handler
**
** hKSeq: Handle of KSeq to terminate.
*/
KSEQ_API BOOL ExitHandler(HKSEQ hKSeq);

/*
** LockExternalAccess - Locks master copy of protocol memory to prevent external changes.
**
** hKSeq: Handle of KSeq.
**
** Note: Must be unlocked with UnlockExternalAccess to allow external changes.
*/
KSEQ_API void LockExternalAccess(HKSEQ hKSeq);

/*
** ReadWordMemory - Reads from master copy of word memory (V). Should be locked with LockExternalAccess prior to reading.
**
** hKSeq: Handle of KSeq.
** Offset: Beginning word location to read.
** Count: Number of words of word memory to read.
** pData: Buffer to receive word memory.
*/
KSEQ_API void ReadWordMemory(HKSEQ hKSeq, DWORD Offset, int Count, void *pData);

/*
** WriteWordMemory - Writes to master copy of word memory. Should be locked with LockExternalAccess prior to writing.
**
** hKSeq: Handle of KSeq.
** Offset: Beginning word location to write.
** Count: Number of words of word memory to write.
** pData: Buffer to provide word memory.
*/
KSEQ_API void WriteWordMemory(HKSEQ hKSeq, DWORD Offset, int Count, void *pData);

/*
** ReadBitMemory - Reads from master copy of bit memory. Should be locked with LockExternalAccess prior to reading.
**
** hKSeq: Handle of KSeq.
** Offset: Beginning byte location to read.
** Count: Number of bytes of bit memory to read.
** pData: Buffer to receive bit memory.
*/
KSEQ_API void ReadBitMemory(HKSEQ hKSeq, DWORD Offset, int Length, void *pData);

/*
** WriteBitMemory - Writes to master copy of bit memory. Should be locked with LockExternalAccess prior to writing.
**
** hKSeq: Handle of KSeq.
** Address: Beginning byte location to write.
** Count: Number of bytes of bit memory to write.
** pData: Buffer to provide bit memory.
*/
KSEQ_API void WriteBitMemory(HKSEQ hKSeq, DWORD Offset, int Length, void *pData);

/*
** GetV - Reads single V memory location from master copy. Should be locked with LockExternalAccess prior to reading.
**
** hKSeq: Handle of KSeq.
** Address: V location to read.
*/
KSEQ_API WORD GetV(HKSEQ hKSeq, int Address);

/*
** SetV - Writes single V memory location to master copy. Should be locked with LockExternalAccess prior to writing.
**
** hKSeq: Handle of KSeq.
** Address: V location to write.
** Val: New value to write.
*/
KSEQ_API void SetV(HKSEQ hKSeq, int Address, WORD Val);

/*
** GetC - Reads single bit C memory location from master copy. Should be locked with LockExternalAccess prior to reading.
**
** hKSeq: Handle of KSeq.
** Address: C location to read. 
*/
KSEQ_API BOOL GetC(HKSEQ hKSeq, int Address);

/*
** SetC - Writes single bit C memory location to master copy. Should be locked with LockExternalAccess prior to writing.
**
** hKSeq: Handle of KSeq.
** Address: C location to write.
** Val: New value to write.
*/
KSEQ_API void SetC(HKSEQ hKSeq, int Address, BOOL Val);

/*
** UnockExternalAccess - Unlocks master copy of protocol memory previously locked with LockExternalAccess.
**
** hKSeq: Handle of KSeq.
*/
KSEQ_API void UnlockExternalAccess(HKSEQ hKSeq);


#ifdef __cplusplus
}
#endif

#endif // __KSEQAPI_H
